@@ -35,6 +35,15 @@
 */
 int gbox_geocentric_slow = LW_FALSE;
 
+/**
+* Utility function for ptarray_contains_point_sphere()
+*/
+static int
+point3d_equals(const POINT3D *p1, const POINT3D *p2)
+{
+	return FP_EQUALS(p1->x, p2->x) && FP_EQUALS(p1->y, p2->y) && FP_EQUALS(p1->z, p2->z);
+}
+
 /**
 * Convert a longitude to the range of -PI,PI
 */
@@ -314,53 +323,75 @@ static int gbox_check_poles(GBOX *gbox)
 	lwfree(gbox_str);
 #endif
 	/* Z axis */
-	if ( gbox->xmin < 0.0 && gbox->xmax > 0.0 &&
-	     gbox->ymin < 0.0 && gbox->ymax > 0.0 )
+	if (gbox->xmin < 0.0 && gbox->xmax > 0.0 &&
+	    gbox->ymin < 0.0 && gbox->ymax > 0.0)
 	{
-		if ( (gbox->zmin + gbox->zmax) > 0.0 )
+		/* Extrema lean positive */
+		if ((gbox->zmin > 0.0) && (gbox->zmax > 0.0))
 		{
 			LWDEBUG(4, "enclosed positive z axis");
 			gbox->zmax = 1.0;
 		}
-		else
+		/* Extrema lean negative */
+		else if ((gbox->zmin < 0.0) && (gbox->zmax < 0.0))
 		{
 			LWDEBUG(4, "enclosed negative z axis");
 			gbox->zmin = -1.0;
 		}
+		/* Extrema both sides! */
+		else
+		{
+			LWDEBUG(4, "enclosed both z axes");
+			gbox->zmin = -1.0;
+			gbox->zmax = 1.0;
+		}
 		rv = LW_TRUE;
 	}
 
 	/* Y axis */
-	if ( gbox->xmin < 0.0 && gbox->xmax > 0.0 &&
-	     gbox->zmin < 0.0 && gbox->zmax > 0.0 )
+	if (gbox->xmin < 0.0 && gbox->xmax > 0.0 &&
+	    gbox->zmin < 0.0 && gbox->zmax > 0.0)
 	{
-		if ( gbox->ymin + gbox->ymax > 0.0 )
+		if ((gbox->ymin > 0.0) && (gbox->ymax > 0.0))
 		{
 			LWDEBUG(4, "enclosed positive y axis");
 			gbox->ymax = 1.0;
 		}
-		else
+		else if ((gbox->ymin < 0.0) && (gbox->ymax < 0.0))
 		{
 			LWDEBUG(4, "enclosed negative y axis");
+			gbox->ymax = -1.0;
+		}
+		else
+		{
+			LWDEBUG(4, "enclosed both y axes");
+			gbox->ymax = 1.0;
 			gbox->ymin = -1.0;
 		}
 		rv = LW_TRUE;
 	}
 
 	/* X axis */
-	if ( gbox->ymin < 0.0 && gbox->ymax > 0.0 &&
-	     gbox->zmin < 0.0 && gbox->zmax > 0.0 )
+	if (gbox->ymin < 0.0 && gbox->ymax > 0.0 &&
+	    gbox->zmin < 0.0 && gbox->zmax > 0.0)
 	{
-		if ( gbox->xmin + gbox->xmax > 0.0 )
+		if ((gbox->xmin > 0.0) && (gbox->xmax > 0.0))
 		{
 			LWDEBUG(4, "enclosed positive x axis");
 			gbox->xmax = 1.0;
 		}
-		else
+		else if ((gbox->xmin < 0.0) && (gbox->xmax < 0.0))
 		{
 			LWDEBUG(4, "enclosed negative x axis");
 			gbox->xmin = -1.0;
 		}
+		else
+		{
+			LWDEBUG(4, "enclosed both x axes");
+			gbox->xmax = 1.0;
+			gbox->xmin = -1.0;
+		}
+
 		rv = LW_TRUE;
 	}
 
@@ -453,7 +484,7 @@ static void vector_difference(const POINT3D *a, const POINT3D *b, POINT3D *n)
 /**
 * Scale a vector out by a factor
 */
-static void vector_scale(POINT3D *n, double scale)
+void vector_scale(POINT3D *n, double scale)
 {
 	n->x *= scale;
 	n->y *= scale;
@@ -1441,28 +1472,84 @@ int edge_calculate_gbox(const POINT3D *A1, const POINT3D *A2, GBOX *gbox)
 	return LW_SUCCESS;
 }
 
-void lwpoly_pt_outside(const LWPOLY *poly, POINT2D *pt_outside)
+/*
+* When we have a globe-covering gbox but we still want an outside
+* point, we do this Very Bad Hack, which is look at the first two points
+* in the ring and then nudge a point to the left of that arc.
+* There is an assumption of convexity built in there, as well as that
+* the shape doesn't have a sharp reversal in it. It's ugly, but
+* it fixes some common cases (large selection polygons) that users
+* are generating. At some point all of geodetic needs a clean-room
+* rewrite.
+* There is also an assumption of CCW exterior ring, which is how the
+* GeoJSON spec defined geographic ring orientation.
+*/
+static int lwpoly_pt_outside_hack(const LWPOLY *poly, POINT2D *pt_outside)
 {
+	GEOGRAPHIC_POINT g1, g2, gSum;
+	POINT4D p1, p2;
+	POINT3D q1, q2, qMid, qCross, qSum;
+	POINTARRAY *pa;
+	if (lwgeom_is_empty((LWGEOM*)poly))
+		return LW_FAILURE;
+	if (poly->nrings < 1)
+		return LW_FAILURE;
+	pa = poly->rings[0];
+	if (pa->npoints < 2)
+		return LW_FAILURE;
+
+	/* First two points of ring */
+	getPoint4d_p(pa, 0, &p1);
+	getPoint4d_p(pa, 1, &p2);
+	/* Convert to XYZ unit vectors */
+	geographic_point_init(p1.x, p1.y, &g1);
+	geographic_point_init(p2.x, p2.y, &g2);
+	geog2cart(&g1, &q1);
+	geog2cart(&g2, &q2);
+	/* Mid-point of first two points */
+	vector_sum(&q1, &q2, &qMid);
+	normalize(&qMid);
+	/* Cross product of first two points (perpendicular) */
+	cross_product(&q1, &q2, &qCross);
+	normalize(&qCross);
+	/* Invert it to put it outside, and scale down */
+	vector_scale(&qCross, -0.2);
+	/* Project midpoint to the right */
+	vector_sum(&qMid, &qCross, &qSum);
+	normalize(&qSum);
+	/* Convert back to lon/lat */
+	cart2geog(&qSum, &gSum);
+	pt_outside->x = rad2deg(gSum.lon);
+	pt_outside->y = rad2deg(gSum.lat);
+	return LW_SUCCESS;
+}
+
+int lwpoly_pt_outside(const LWPOLY *poly, POINT2D *pt_outside)
+{
+	int rv;
 	/* Make sure we have boxes */
 	if ( poly->bbox )
 	{
-		gbox_pt_outside(poly->bbox, pt_outside);
-		return;
+		rv = gbox_pt_outside(poly->bbox, pt_outside);
 	}
 	else
 	{
 		GBOX gbox;
 		lwgeom_calculate_gbox_geodetic((LWGEOM*)poly, &gbox);
-		gbox_pt_outside(&gbox, pt_outside);
-		return;
+		rv = gbox_pt_outside(&gbox, pt_outside);
 	}
+
+	if (rv == LW_FALSE)
+		return lwpoly_pt_outside_hack(poly, pt_outside);
+
+	return rv;
 }
 
 /**
 * Given a unit geocentric gbox, return a lon/lat (degrees) coordinate point point that is
 * guaranteed to be outside the box (and therefore anything it contains).
 */
-void gbox_pt_outside(const GBOX *gbox, POINT2D *pt_outside)
+int gbox_pt_outside(const GBOX *gbox, POINT2D *pt_outside)
 {
 	double grow = M_PI / 180.0 / 60.0; /* one arc-minute */
 	int i;
@@ -1529,7 +1616,7 @@ void gbox_pt_outside(const GBOX *gbox, POINT2D *pt_outside)
 				pt_outside->x = rad2deg(g.lon);
 				pt_outside->y = rad2deg(g.lat);
 				LWDEBUGF(4, "returning POINT(%.8g %.8g) as outside point", pt_outside->x, pt_outside->y);
-				return;
+				return LW_SUCCESS;
 			}
 		}
 
@@ -1538,8 +1625,8 @@ void gbox_pt_outside(const GBOX *gbox, POINT2D *pt_outside)
 	}
 
 	/* This should never happen! */
-	lwerror("BOOM! Could not generate outside point!");
-	return;
+	// lwerror("BOOM! Could not generate outside point!");
+	return LW_FAILURE;
 }
 
 
@@ -2464,7 +2551,7 @@ int lwpoly_covers_point2d(const LWPOLY *poly, const POINT2D *pt_to_test)
 	}
 
 	/* Calculate our outside point from the gbox */
-	gbox_pt_outside(&gbox, &pt_outside);
+	lwpoly_pt_outside(poly, &pt_outside);
 
 	LWDEBUGF(4, "pt_outside POINT(%.18g %.18g)", pt_outside.x, pt_outside.y);
 	LWDEBUGF(4, "pt_to_test POINT(%.18g %.18g)", pt_to_test->x, pt_to_test->y);
@@ -2798,6 +2885,7 @@ int getPoint2d_p_ro(const POINTARRAY *pa, uint32_t n, POINT2D **point)
 	return LW_SUCCESS;
 }
 
+
 int ptarray_calculate_gbox_geodetic(const POINTARRAY *pa, GBOX *gbox)
 {
 	uint32_t i;
@@ -3373,6 +3461,10 @@ point_in_cone(const POINT3D *A1, const POINT3D *A2, const POINT3D *P)
 	POINT3D AC; /* Center point of A1/A2 */
 	double min_similarity, similarity;
 
+	/* Boundary case */
+	if (point3d_equals(A1, P) || point3d_equals(A2, P))
+		return LW_TRUE;
+
 	/* The normalized sum bisects the angle between start and end. */
 	vector_sum(A1, A2, &AC);
 	normalize(&AC);
@@ -3380,27 +3472,51 @@ point_in_cone(const POINT3D *A1, const POINT3D *A2, const POINT3D *P)
 	/* The projection of start onto the center defines the minimum similarity */
 	min_similarity = dot_product(A1, &AC);
 
-	/* The projection of candidate p onto the center */
-	similarity = dot_product(P, &AC);
-
-	/* If the point is more similar than the end, the point is in the cone */
-	if ( similarity > min_similarity || fabs(similarity - min_similarity) < 2e-16 )
+	/* If the edge is sufficiently curved, use the dot product test */
+	if (fabs(1.0 - min_similarity) > 1e-10)
 	{
-		return LW_TRUE;
+		/* The projection of candidate p onto the center */
+		similarity = dot_product(P, &AC);
+
+		/* If the projection of the candidate is larger than */
+		/* the projection of the start point, the candidate */
+		/* must be closer to the center than the start, so */
+		/* therefor inside the cone */
+		if (similarity > min_similarity)
+		{
+			return LW_TRUE;
+		}
+		else
+		{
+			return LW_FALSE;
+		}
+	}
+	else
+	{
+		/* Where the edge is very narrow, the dot product test */
+		/* fails, but we can use the almost-planar nature of the */
+		/* problem space then to test if the vector from the */
+		/* candidate to the start point in a different direction */
+		/* to the vector from candidate to end point */
+		/* If so, then candidate is between start and end */
+		POINT3D PA1, PA2;
+		vector_difference(P, A1, &PA1);
+		vector_difference(P, A2, &PA2);
+		normalize(&PA1);
+		normalize(&PA2);
+		if (dot_product(&PA1, &PA2) < 0.0)
+		{
+			return LW_TRUE;
+		}
+		else
+		{
+			return LW_FALSE;
+		}
 	}
 	return LW_FALSE;
 }
 
 
-/**
-* Utility function for ptarray_contains_point_sphere()
-*/
-static int
-point3d_equals(const POINT3D *p1, const POINT3D *p2)
-{
-	return FP_EQUALS(p1->x, p2->x) && FP_EQUALS(p1->y, p2->y) && FP_EQUALS(p1->z, p2->z);
-}
-
 /**
 * Utility function for edge_intersects(), signum with a tolerance
 * in determining if the value is zero.
